package net.krakatoa.proxy.listener;

import com.mongodb.client.model.Filters;
import com.mongodb.client.model.UpdateOptions;
import com.mongodb.client.model.Updates;
import earth.krakatao.protocol.KraSocketClientProtocolDest;
import earth.krakatao.protocol.KraSocketClientProtocolMessage;
import earth.krakatao.protocol.KraSocketClientProtocolStatus;
import java.security.SecureRandom;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import net.krakatoa.proxy.ProxySystem;
import net.md_5.bungee.api.ProxyServer;
import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.HoverEvent;
import net.md_5.bungee.api.chat.HoverEvent.Action;
import net.md_5.bungee.api.chat.TextComponent;
import net.md_5.bungee.api.chat.hover.content.Text;
import net.md_5.bungee.api.config.ServerInfo;
import net.md_5.bungee.api.connection.ProxiedPlayer;
import net.md_5.bungee.api.event.PostLoginEvent;
import net.md_5.bungee.api.event.ServerSwitchEvent;
import net.md_5.bungee.api.plugin.Listener;
import net.md_5.bungee.event.EventHandler;
import org.bson.Document;
import org.bson.conversions.Bson;

public class PostLoginListener implements Listener {

  @EventHandler
  public void onPostLogin(PostLoginEvent event) {
    ProxiedPlayer proxiedPlayer = event.getPlayer();

    String uuid = ProxySystem.getInstance().getFormatter()
        .formatUuid(proxiedPlayer.getUniqueId().toString());

    ProxySystem.getInstance().getPlayers()
        .find(Filters.eq("uuid", uuid))
        .first((document, throwable) -> {
          if (document == null) {
            // create player in db and send voice url
            System.out.println("create player in db");
            String voiceWebCode = generateVoiceWebCode(uuid);

            document = new Document("uuid", uuid).append("voiceWebCode", voiceWebCode);

            ProxySystem.getInstance().getPlayers().insertOne(document, (unused, throwable1) -> {
              System.out.println("insertOne");
            });

            sendVoiceWebCodeUrl(proxiedPlayer, voiceWebCode);
          } else {
            // check if both sockets connected
            int cmdID = ProxySystem.getInstance().getKraSocketClientProtocol().generateCmdID();

            Document finalDocument = document;
            Consumer<KraSocketClientProtocolMessage> consumer = value -> {
              if (value.getArgs().equals("1")) {
                // connected! Send player to game server
                System.out.println(
                    proxiedPlayer.getName() + " sockets connected! Send player to game server");
                proxiedPlayer.sendMessage(
                    new TextComponent("§bWillkommen zurück §6" + proxiedPlayer.getDisplayName()));

                ProxyServer.getInstance().getScheduler().schedule(ProxySystem.getInstance(), () -> {
                  ServerInfo serverInfos = ProxyServer.getInstance().getServerInfo("NewYork-1");
                  proxiedPlayer.connect(serverInfos);
                }, 2L, TimeUnit.SECONDS);
              } else {
                // not connected! Send voice url
                System.out.println(
                    proxiedPlayer.getName() + " sockets not connected! Send voice url");

                String voiceWebCode = generateVoiceWebCode(uuid);

                Bson updates = Updates.combine(Updates.set("voiceWebCode", voiceWebCode));

                UpdateOptions options = new UpdateOptions().upsert(true);

                ProxySystem.getInstance().getPlayers()
                    .updateOne(finalDocument, updates, options, (updateResult, throwable1) -> {
                    });

                System.out.println("voiceWebCode " + voiceWebCode);

                sendVoiceWebCodeUrl(proxiedPlayer, voiceWebCode);
              }

              System.out.println(
                  "consumer response value " + value.getUuid() + " " + value.getCmdID());
            };

            ProxySystem.getInstance().getKraSocketClient().getSocketClient()
                .SendMessage(new KraSocketClientProtocolMessage(
                    KraSocketClientProtocolStatus.GET.getStatus(), cmdID,
                    KraSocketClientProtocolDest.BACKEND.getStatus(), proxiedPlayer.getUniqueId(),
                    (short) 10, "", consumer));
          }
        });
  }

  public void sendVoiceWebCodeUrl(ProxiedPlayer proxiedPlayer, String voiceWebCode) {
    String url = ProxySystem.getInstance().getConfigHandler()
        .getWebVoiceAddress() + voiceWebCode;

    TextComponent textComponent = new TextComponent(
        "Klicke auf den Link, um dich mit dem Voice Server zu verbinden");

    TextComponent linkComponent = new TextComponent(url);
    linkComponent.setHoverEvent(new HoverEvent(Action.SHOW_TEXT,
        new Text(url)));
    linkComponent.setClickEvent(
        new ClickEvent(ClickEvent.Action.OPEN_URL,
            url));

    textComponent.addExtra("\n");
    textComponent.addExtra(linkComponent);

    proxiedPlayer.sendMessage(textComponent);
  }

  public String generateVoiceWebCode(String uuid) {
    SecureRandom secureRandom = new SecureRandom();

    String chrs = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

    String customTag = secureRandom.ints(8, 0, chrs.length()).mapToObj(chrs::charAt)
        .collect(StringBuilder::new, StringBuilder::append, StringBuilder::append).toString();

    System.out.println(customTag);


   /* String chrs = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    SecureRandom secureRandom = SecureRandom.getInstanceStrong();

    String customTag = secureRandom.ints(9, 0, chrs.length()).mapToObj(chrs::charAt)
        .collect(StringBuilder::new, StringBuilder::append, StringBuilder::append).toString();
    System.out.println(customTag);

    /*AtomicBoolean avail = new AtomicBoolean(false);

    while (!avail.get()) {
      System.out.println("while");
      secureRandom.ints(8, 0, chrs.length()).mapToObj(chrs::charAt)
          .collect(StringBuilder::new, StringBuilder::append, StringBuilder::append).toString();
      System.out.println(customTag);

      ProxySystem.getInstance().getPlayerRegistrationProcess().find(Filters.eq("webCode", chrs))
          .first((document, throwable) -> {
            if (document == null) {
              avail.set(true);
            }
          });
    }*/

    return customTag;
  }

  @EventHandler
  public void onServerSwitch(ServerSwitchEvent event) {
    ProxiedPlayer player = event.getPlayer();

    player.sendMessage(
        new TextComponent("Switched server " + player.getServer().getInfo().getName()));
  }

  /*

  @EventHandler
  public void onServerConnect(ServerConnectEvent event) {
    System.out.println(
        "REASON " + event.getReason() + " " + event.getPlayer().getUniqueId().toString());

    if (!event.getReason().equals(Reason.JOIN_PROXY)) {
      System.out.println("return join proxy");
      return;
    }

  //  event.setTarget(ProxyServer.getInstance().getServerInfo("NewYork-1"));
/*
    ProxySystem.getInstance().getKraProtocol()
        .SendMessage(11, 10, 0, event.getPlayer().getUniqueId().toString().replaceAll("-", ""), 10,
            "");

    //event.setTarget(ProxyServer.getInstance().getServerInfo("NewYork-1"));
  } */
}
